@@ -25,11 +25,12 @@
 # established at this point, and setup.py will use a version of next-$(revno).
 # If the releaselevel is 'final', then the tarball will be major.minor.micro.
 # Otherwise it is major.minor.micro~$(revno).
-__version__ = (0, 0, 12, 'final', 0)
+__version__ = (0, 0, 13, 'final', 0)
 
 __all__ = [
     'Config',
     'convert_result_to_list',
+    'pprint_to_stream',
     'publish_new_only',
     'publish_to_many',
     'publish_with_fallback',
@@ -38,6 +39,7 @@ __all__ = [
 from oops.config import Config
 from oops.publishers import (
     convert_result_to_list,
+    pprint_to_stream,
     publish_new_only,
     publish_to_many,
     publish_with_fallback,
