@@ -24,7 +24,7 @@ Typical usage:
   >>> config = Config()
   >>> def demo_publish(report):
   ...    return 'id 1'
-  >>> config.publishers.append(demo_publish)
+  >>> config.publisher = demo_publish
 
   This allows aggregation of oops reports from different programs into one
   oops-tools install.
@@ -100,33 +100,46 @@ __all__ = [
 __metaclass__ = type
 
 from copy import deepcopy
+import warnings
 
 from createhooks import default_hooks
+from publishers import (
+    convert_result_to_list,
+    publish_to_many,
+    )
 
 
 class Config:
     """The configuration for the OOPS system.
-    
+
     :ivar on_create: A list of callables to call when making a new report. Each
         will be called in series with the new report and a creation context
         dict. The return value of the callbacks is ignored.
     :ivar filters: A list of callables to call when filtering a report. Each
         will be called in series with a report that is about to be published.
-        If the filter returns true (that is not None, 0, '' or False), then 
+        If the filter returns true (that is not None, 0, '' or False), then
         the report will not be published, and the call to publish will return
         None to the user.
+    :ivar publisher: A callable to call when publishing a report.
+        It will be called in series with the report to publish. It is expected
+        to return a list of ids. See the publish() method for more
+        information.
     :ivar publishers: A list of callables to call when publishing a report.
         Each will be called in series with the report to publish. Their return
         value will be assigned to the reports 'id' key : if a publisher
         allocates a different id than a prior publisher, only the last
         publisher in the list will have its id present in the report at the
-        end. See the publish() method for more information.
+        end. See the publish() method for more information. This attribute
+        is deprecated, Use the `publisher` attribute instead, and see
+        `oops.publishers.publish_to_many` if you want to publish to multiple
+        publishers.
     """
 
     def __init__(self):
         self.filters = []
         self.on_create = list(default_hooks)
         self.template = {}
+        self.publisher = None
         self.publishers = []
 
     def create(self, context=None):
@@ -153,43 +166,54 @@ class Config:
     def publish(self, report):
         """Publish a report.
 
-        Each publisher is passed the report to publish. Publishers should
-        return the id they allocated-or-used for the report, which gets
-        automatically put into the report for them. All of the ids are also
-        returned to the caller, allowing them to handle the case when multiple
-        publishers allocated different ids. The id from the last publisher is
-        left in the report's id key. This is done for three reasons:
-        
-        * As a convenience for the common case when only one publisher is
-          present.
-          
-        * This allows the next publisher in the chain to reuse that id if it
-          wishes.
-          
-        * Showing users that a report was made only needs one id and this means
-          other code doesn't need to arbitrarily pick one - it is done by
-          publish().
+        The report will be passed through any filters, and then handed
+        off to the callable assigned to the `publisher` instance variable,
+        if any, and then passed through any publishers in the `publishers`
+        list instance variable. The return value will be the list returned
+        by `publisher` method, with the ids returned by the `publishers`
+        appended. The `publishers` list is deprecated, and the `publisher`
+        attribute should be used instead, with
+        `oops.publishers.publish_to_many` used if needed.
 
-        If a publisher raises an exception, that will propagate to the caller.
-        If a publisher returns anything non-True (that is None, 0, False, ''),
-        it indicates that the publisher did not publish the report and no change
-        will be made to the report's 'id' key for that publisher.
-        This permits fallback chains. For instance the first publisher may
-        choose not to publish a report (e.g. the place it would publish to is
-        offline) and the second publisher can detect that by checking if there
-        is an id key.
+        The `publisher` should return a list of ids that were
+        allocated-or-used for the report. The `publishers` should each return
+        any id that was allocated-or-used for the report. The return value of
+        the callables in the `publishers` list will be assigned to the `id`
+        key of the report. If a publisher in the `publishers` list returns
+        anything non-True (that is None, 0, False, ''), it indicates that the
+        publisher did not publish the report.
+
+        The last entry in the list of ids, if any, will be assigned to the 'id'
+        key of the report before returning, so that clients that only care to
+        deal with one id don't have to pick one themselves.
+
+        The whole list of ids will be returned to the caller to allow them
+        to handle the case where multiple ids were used for a report.
+
+        If any publisher raises an exception, that will propagate to the caller.
 
         :return: A list of the allocated ids.
         """
         for report_filter in self.filters:
             if report_filter(report):
                 return None
-        result = []
-        for publisher in self.publishers:
-            id = publisher(report)
-            if id:
-                report['id'] = id
-                result.append(id)
-            else:
-                result.append(None)
-        return result
+        # XXX: james_w 2012-06-19 bug=1015293: Deprecated code path,
+        #       this should be removed once users have had a chance
+        #       to migrate. The constructor and docstrings should
+        #       also be cleaned up at the same time.
+        if self.publishers:
+            warnings.warn(
+                "Using the oops.Config.publishers attribute is "
+                "deprecated. Use the oops.Config.publisher attribute "
+                "instead, with an oops.publishers.publish_to_many object "
+                "if multiple publishers are needed",
+                DeprecationWarning, stacklevel=2)
+        old_publishers = map(convert_result_to_list, self.publishers)
+        if self.publisher:
+            publisher = publish_to_many(self.publisher, *old_publishers)
+        else:
+            publisher = publish_to_many(*old_publishers)
+        ret = publisher(report)
+        if ret:
+            report['id'] = ret[-1]
+        return ret
